@@ -7,9 +7,11 @@ makedocs(; modules=[KrylovKit],
          pages=["Home" => "index.md",
                 "Manual" => ["man/intro.md",
                              "man/linear.md",
+                             "man/leastsquares.md",
                              "man/eig.md",
                              "man/svd.md",
                              "man/matfun.md",
+                             "man/reallinear.md",
                              "man/algorithms.md",
                              "man/implementation.md"]],
          format=Documenter.HTML(; prettyurls=get(ENV, "CI", nothing) == "true"))
