@@ -84,8 +84,12 @@ abstract type KrylovAlgorithm end
 
 # General purpose; good for linear systems, eigensystems and matrix functions
 """
-    Lanczos(; krylovdim = KrylovDefaults.krylovdim, maxiter = KrylovDefaults.maxiter,
-        tol = KrylovDefaults.tol, orth = KrylovDefaults.orth, eager = false, verbosity = 0)
+    Lanczos(; krylovdim=KrylovDefaults.krylovdim[],
+            maxiter=KrylovDefaults.maxiter[],
+            tol=KrylovDefaults.tol[],
+            orth=KrylovDefaults.orth,
+            eager=false,
+            verbosity=KrylovDefaults.verbosity[])
 
 Represents the Lanczos algorithm for building the Krylov subspace; assumes the linear
 operator is real symmetric or complex Hermitian. Can be used in `eigsolve` and
@@ -93,10 +97,11 @@ operator is real symmetric or complex Hermitian. Can be used in `eigsolve` and
 `krylovdim`, which will be repeated at most `maxiter` times and will stop when the norm of
 the residual of the Lanczos factorization is smaller than `tol`. The orthogonalizer `orth`
 will be used to orthogonalize the different Krylov vectors. Eager mode, as selected by
-`eager = true`, means that the algorithm that uses this Lanczos process (e.g. `eigsolve`)
+`eager=true`, means that the algorithm that uses this Lanczos process (e.g. `eigsolve`)
 can try to finish its computation before the total Krylov subspace of dimension `krylovdim`
-is constructed. Default verbosity level `verbosity` is zero, meaning that no output will be
-printed.
+is constructed. The default verbosity level `verbosity` amounts to printing warnings upon
+lack of convergence.
+
 
 Use `Arnoldi` for non-symmetric or non-Hermitian linear operators.
 
@@ -111,26 +116,31 @@ struct Lanczos{O<:Orthogonalizer,S<:Real} <: KrylovAlgorithm
     verbosity::Int
 end
 function Lanczos(;
-                 krylovdim::Int=KrylovDefaults.krylovdim,
-                 maxiter::Int=KrylovDefaults.maxiter,
-                 tol::Real=KrylovDefaults.tol,
+                 krylovdim::Int=KrylovDefaults.krylovdim[],
+                 maxiter::Int=KrylovDefaults.maxiter[],
+                 tol::Real=KrylovDefaults.tol[],
                  orth::Orthogonalizer=KrylovDefaults.orth,
                  eager::Bool=false,
-                 verbosity::Int=0)
+                 verbosity::Int=KrylovDefaults.verbosity[])
     return Lanczos(orth, krylovdim, maxiter, tol, eager, verbosity)
 end
 
 """
-    GKL(; krylovdim = KrylovDefaults.krylovdim, maxiter = KrylovDefaults.maxiter,
-        tol = KrylovDefaults.tol, orth = KrylovDefaults.orth, verbosity = 0)
+    GKL(; krylovdim=KrylovDefaults.krylovdim[],
+        maxiter=KrylovDefaults.maxiter[],
+        tol=KrylovDefaults.tol[],
+        orth=KrylovDefaults.orth,
+        eager=false,
+        verbosity=KrylovDefaults.verbosity[])
 
 Represents the Golub-Kahan-Lanczos bidiagonalization algorithm for sequentially building a
 Krylov-like factorization of a general matrix or linear operator with a bidiagonal reduced
 matrix. Can be used in `svdsolve`. The corresponding algorithm builds a Krylov subspace of
 size at most `krylovdim`, which will be repeated at most `maxiter` times and will stop when
 the norm of the residual of the Arnoldi factorization is smaller than `tol`. The
-orthogonalizer `orth` will be used to orthogonalize the different Krylov vectors. Default
-verbosity level `verbosity` is zero, meaning that no output will be printed.
+orthogonalizer `orth` will be used to orthogonalize the different Krylov vectors. The default
+verbosity level `verbosity` amounts to printing warnings upon lack of convergence.
+
 
 See also: [`svdsolve`](@ref), [`Orthogonalizer`](@ref)
 """
@@ -143,28 +153,33 @@ struct GKL{O<:Orthogonalizer,S<:Real} <: KrylovAlgorithm
     verbosity::Int
 end
 function GKL(;
-             krylovdim::Int=KrylovDefaults.krylovdim,
-             maxiter::Int=KrylovDefaults.maxiter,
-             tol::Real=KrylovDefaults.tol,
+             krylovdim::Int=KrylovDefaults.krylovdim[],
+             maxiter::Int=KrylovDefaults.maxiter[],
+             tol::Real=KrylovDefaults.tol[],
              orth::Orthogonalizer=KrylovDefaults.orth,
              eager::Bool=false,
-             verbosity::Int=0)
+             verbosity::Int=KrylovDefaults.verbosity[])
     return GKL(orth, krylovdim, maxiter, tol, eager, verbosity)
 end
 
 """
-    Arnoldi(; krylovdim = KrylovDefaults.krylovdim, maxiter = KrylovDefaults.maxiter,
-        tol = KrylovDefaults.tol, orth = KrylovDefaults.orth, eager = false, verbosity = 0)
+    Arnoldi(; krylovdim=KrylovDefaults.krylovdim[],
+            maxiter=KrylovDefaults.maxiter[],
+            tol=KrylovDefaults.tol[],
+            orth=KrylovDefaults.orth,
+            eager=false,
+            verbosity=KrylovDefaults.verbosity[])
 
 Represents the Arnoldi algorithm for building the Krylov subspace for a general matrix or
 linear operator. Can be used in `eigsolve` and `exponentiate`. The corresponding algorithms
 will build a Krylov subspace of size at most `krylovdim`, which will be repeated at most
 `maxiter` times and will stop when the norm of the residual of the Arnoldi factorization is
 smaller than `tol`. The orthogonalizer `orth` will be used to orthogonalize the different
-Krylov vectors. Eager mode, as selected by `eager = true`, means that the algorithm that
+Krylov vectors. Eager mode, as selected by `eager=true`, means that the algorithm that
 uses this Arnoldi process (e.g. `eigsolve`) can try to finish its computation before the
-total Krylov subspace of dimension `krylovdim` is constructed. Default verbosity level
-`verbosity` is zero, meaning that no output will be printed.
+total Krylov subspace of dimension `krylovdim` is constructed. The default verbosity level
+`verbosity` amounts to printing warnings upon lack of convergence.
+
 
 Use `Lanczos` for real symmetric or complex Hermitian linear operators.
 
@@ -180,18 +195,22 @@ struct Arnoldi{O<:Orthogonalizer,S<:Real} <: KrylovAlgorithm
     verbosity::Int
 end
 function Arnoldi(;
-                 krylovdim::Int=KrylovDefaults.krylovdim,
-                 maxiter::Int=KrylovDefaults.maxiter,
-                 tol::Real=KrylovDefaults.tol,
+                 krylovdim::Int=KrylovDefaults.krylovdim[],
+                 maxiter::Int=KrylovDefaults.maxiter[],
+                 tol::Real=KrylovDefaults.tol[],
                  orth::Orthogonalizer=KrylovDefaults.orth,
                  eager::Bool=false,
-                 verbosity::Int=0)
+                 verbosity::Int=KrylovDefaults.verbosity[])
     return Arnoldi(orth, krylovdim, maxiter, tol, eager, verbosity)
 end
 
 """
-    GolubYe(; krylovdim = KrylovDefaults.krylovdim, maxiter = KrylovDefaults.maxiter,
-        tol = KrylovDefaults.tol, orth = KrylovDefaults.orth, verbosity = 0)
+    GolubYe(; krylovdim=KrylovDefaults.krylovdim[],
+            maxiter=KrylovDefaults.maxiter[],
+            tol=KrylovDefaults.tol[],
+            orth=KrylovDefaults.orth,
+            eager=false,
+            verbosity=KrylovDefaults.verbosity[])
 
 Represents the Golub-Ye algorithm for solving hermitian (symmetric) generalized eigenvalue
 problems `A x = λ B x` with positive definite `B`, without the need for inverting `B`.
@@ -199,7 +218,7 @@ Builds a Krylov subspace of size `krylovdim` starting from an estimate `x` by ac
 `(A - ρ(x) B)`, where `ρ(x) = dot(x, A*x)/dot(x, B*x)`, and employing the Lanczos
 algorithm. This process is repeated at most `maxiter` times. In every iteration `k>1`, the
 subspace will also be expanded to size `krylovdim+1` by adding ``x_k - x_{k-1}``, which is
-known as the LOPCG correction and was suggested by Money and Ye. With `krylovdim = 2`, this
+known as the LOPCG correction and was suggested by Money and Ye. With `krylovdim=2`, this
 algorithm becomes equivalent to `LOPCG`.
 """
 struct GolubYe{O<:Orthogonalizer,S<:Real} <: KrylovAlgorithm
@@ -210,11 +229,11 @@ struct GolubYe{O<:Orthogonalizer,S<:Real} <: KrylovAlgorithm
     verbosity::Int
 end
 function GolubYe(;
-                 krylovdim::Int=KrylovDefaults.krylovdim,
-                 maxiter::Int=KrylovDefaults.maxiter,
-                 tol::Real=KrylovDefaults.tol,
+                 krylovdim::Int=KrylovDefaults.krylovdim[],
+                 maxiter::Int=KrylovDefaults.maxiter[],
+                 tol::Real=KrylovDefaults.tol[],
                  orth::Orthogonalizer=KrylovDefaults.orth,
-                 verbosity::Int=0)
+                 verbosity::Int=KrylovDefaults.verbosity[])
     return GolubYe(orth, krylovdim, maxiter, tol, verbosity)
 end
 
@@ -222,16 +241,17 @@ end
 abstract type LinearSolver <: KrylovAlgorithm end
 
 """
-    CG(; maxiter = KrylovDefaults.maxiter, tol = KrylovDefaults.tol)
+    CG(; maxiter=KrylovDefaults.maxiter[], tol=KrylovDefaults.tol[], verbosity=KrylovDefaults.verbosity[])
 
 Construct an instance of the conjugate gradient algorithm with specified parameters, which
 can be passed to `linsolve` in order to iteratively solve a linear system with a positive
 definite (and thus symmetric or hermitian) coefficient matrix or operator. The `CG` method
 will search for the optimal `x` in a Krylov subspace of maximal size `maxiter`, or stop when
-`norm(A*x - b) < tol`. Default verbosity level `verbosity` is zero, meaning that no output
-will be printed.
+`norm(A*x - b) < tol`. The default verbosity level `verbosity` amounts to printing warnings
+upon lack of convergence.
 
-See also: [`linsolve`](@ref), [`MINRES`](@ref), [`GMRES`](@ref), [`BiCG`](@ref),
+
+See also: [`linsolve`](@ref), [`MINRES`](@ref), [`GMRES`](@ref), [`BiCG`](@ref), [`LSMR`](@ref),
 [`BiCGStab`](@ref)
 """
 struct CG{S<:Real} <: LinearSolver
@@ -240,29 +260,33 @@ struct CG{S<:Real} <: LinearSolver
     verbosity::Int
 end
 function CG(;
-            maxiter::Integer=KrylovDefaults.maxiter,
-            tol::Real=KrylovDefaults.tol,
-            verbosity::Int=0)
+            maxiter::Integer=KrylovDefaults.maxiter[],
+            tol::Real=KrylovDefaults.tol[],
+            verbosity::Int=KrylovDefaults.verbosity[])
     return CG(maxiter, tol, verbosity)
 end
 
 """
-    GMRES(; krylovdim = KrylovDefaults.krylovdim, maxiter = KrylovDefaults.maxiter,
-        tol = KrylovDefaults.tol, orth::Orthogonalizer = KrylovDefaults.orth)
+    GMRES(; krylovdim=KrylovDefaults.krylovdim[],
+            maxiter=KrylovDefaults.maxiter[],
+            tol=KrylovDefaults.tol[], 
+            orth::Orthogonalizer=KrylovDefaults.orth,
+            verbosity=KrylovDefaults.verbosity[])
 
 Construct an instance of the GMRES algorithm with specified parameters, which can be passed
 to `linsolve` in order to iteratively solve a linear system. The `GMRES` method will search
 for the optimal `x` in a Krylov subspace of maximal size `krylovdim`, and repeat this
 process for at most `maxiter` times, or stop when `norm(A*x - b) < tol`. In building the
-Krylov subspace, `GMRES` will use the orthogonalizer `orth`. Default verbosity level
-`verbosity` is zero, meaning that no output will be printed.
+Krylov subspace, `GMRES` will use the orthogonalizer `orth`. The default verbosity level
+`verbosity` amounts to printing warnings upon lack of convergence.
+
 
 Note that in the traditional nomenclature of `GMRES`, the parameter `krylovdim` is referred
 to as the restart parameter, and `maxiter` is the number of outer iterations, i.e. restart
 cycles. The total iteration count, i.e. the number of expansion steps, is roughly
 `krylovdim` times the number of iterations.
 
-See also: [`linsolve`](@ref), [`BiCG`](@ref), [`BiCGStab`](@ref), [`CG`](@ref),
+See also: [`linsolve`](@ref), [`BiCG`](@ref), [`BiCGStab`](@ref), [`CG`](@ref), [`LSMR`](@ref),
 [`MINRES`](@ref)
 """
 struct GMRES{O<:Orthogonalizer,S<:Real} <: LinearSolver
@@ -273,17 +297,17 @@ struct GMRES{O<:Orthogonalizer,S<:Real} <: LinearSolver
     verbosity::Int
 end
 function GMRES(;
-               krylovdim::Integer=KrylovDefaults.krylovdim,
-               maxiter::Integer=KrylovDefaults.maxiter,
-               tol::Real=KrylovDefaults.tol,
+               krylovdim::Integer=KrylovDefaults.krylovdim[],
+               maxiter::Integer=KrylovDefaults.maxiter[],
+               tol::Real=KrylovDefaults.tol[],
                orth::Orthogonalizer=KrylovDefaults.orth,
-               verbosity::Int=0)
+               verbosity::Int=KrylovDefaults.verbosity[])
     return GMRES(orth, maxiter, krylovdim, tol, verbosity)
 end
 
 # TODO
 """
-    MINRES(; maxiter = KrylovDefaults.maxiter, tol = KrylovDefaults.tol)
+    MINRES(; maxiter=KrylovDefaults.maxiter[], tol=KrylovDefaults.tol[], verbosity=KrylovDefaults.verbosity[])
 
     !!! warning "Not implemented yet"
 
@@ -292,10 +316,11 @@ end
     real symmetric or complex hermitian coefficient matrix or operator. The `MINRES` method
     will search for the optimal `x` in a Krylov subspace of maximal size `maxiter`, or stop
     when `norm(A*x - b) < tol`. In building the Krylov subspace, `MINRES` will use the
-    orthogonalizer `orth`. Default verbosity level `verbosity` is zero, meaning that no
-    output will be printed.
+    orthogonalizer `orth`. The default verbosity level `verbosity` amounts to printing
+    warnings upon lack of convergence.
 
-See also: [`linsolve`](@ref), [`CG`](@ref), [`GMRES`](@ref), [`BiCG`](@ref),
+
+See also: [`linsolve`](@ref), [`CG`](@ref), [`GMRES`](@ref), [`BiCG`](@ref), [`LSMR`](@ref),
 [`BiCGStab`](@ref)
 """
 struct MINRES{S<:Real} <: LinearSolver
@@ -304,14 +329,14 @@ struct MINRES{S<:Real} <: LinearSolver
     verbosity::Int
 end
 function MINRES(;
-                maxiter::Integer=KrylovDefaults.maxiter,
-                tol::Real=KrylovDefaults.tol,
-                verbosity::Int=0)
+                maxiter::Integer=KrylovDefaults.maxiter[],
+                tol::Real=KrylovDefaults.tol[],
+                verbosity::Int=KrylovDefaults.verbosity[])
     return MINRES(maxiter, tol, verbosity)
 end
 
 """
-    BiCG(; maxiter = KrylovDefaults.maxiter, tol = KrylovDefaults.tol)
+    BiCG(; maxiter=KrylovDefaults.maxiter[], tol=KrylovDefaults.tol[], verbosity=KrylovDefaults.verbosity[])
 
     !!! warning "Not implemented yet"
 
@@ -319,10 +344,11 @@ end
     which can be passed to `linsolve` in order to iteratively solve a linear system general
     linear map, of which the adjoint can also be applied. The `BiCG` method will search for
     the optimal `x` in a Krylov subspace of maximal size `maxiter`, or stop when `norm(A*x -
-    b) < tol`. Default verbosity level `verbosity` is zero, meaning that no output will be
-    printed.
+    b) < tol`. The default verbosity level `verbosity` amounts to printing warnings upon
+    lack of convergence.
 
-See also: [`linsolve`](@ref), [`GMRES`](@ref), [`CG`](@ref), [`BiCGStab`](@ref),
+
+See also: [`linsolve`](@ref), [`GMRES`](@ref), [`CG`](@ref), [`BiCGStab`](@ref), [`LSMR`](@ref),
 [`MINRES`](@ref)
 """
 struct BiCG{S<:Real} <: LinearSolver
@@ -331,22 +357,22 @@ struct BiCG{S<:Real} <: LinearSolver
     verbosity::Int
 end
 function BiCG(;
-              maxiter::Integer=KrylovDefaults.maxiter,
-              tol::Real=KrylovDefaults.tol,
-              verbosity::Int=0)
+              maxiter::Integer=KrylovDefaults.maxiter[],
+              tol::Real=KrylovDefaults.tol[],
+              verbosity::Int=KrylovDefaults.verbosity[])
     return BiCG(maxiter, tol, verbosity)
 end
 
 """
-    BiCGStab(; maxiter = KrylovDefaults.maxiter, tol = KrylovDefaults.tol)
+    BiCGStab(; maxiter=KrylovDefaults.maxiter[], tol=KrylovDefaults.tol[], verbosity=KrylovDefaults.verbosity[])
 
     Construct an instance of the Biconjugate gradient algorithm with specified parameters,
     which can be passed to `linsolve` in order to iteratively solve a linear system general
     linear map. The `BiCGStab` method will search for the optimal `x` in a Krylov subspace
-    of maximal size `maxiter`, or stop when `norm(A*x - b) < tol`. Default verbosity level
-    `verbosity` is zero, meaning that no output will be printed.
+    of maximal size `maxiter`, or stop when `norm(A*x - b) < tol`. The default verbosity level 
+    `verbosity` amounts to printing warnings upon lack of convergence.
 
-See also: [`linsolve`](@ref), [`GMRES`](@ref), [`CG`](@ref), [`BiCG`](@ref),
+See also: [`linsolve`](@ref), [`GMRES`](@ref), [`CG`](@ref), [`BiCG`](@ref), [`LSMR`](@ref),
 [`MINRES`](@ref)
 """
 struct BiCGStab{S<:Real} <: LinearSolver
@@ -355,12 +381,52 @@ struct BiCGStab{S<:Real} <: LinearSolver
     verbosity::Int
 end
 function BiCGStab(;
-                  maxiter::Integer=KrylovDefaults.maxiter,
-                  tol::Real=KrylovDefaults.tol,
-                  verbosity::Int=0)
+                  maxiter::Integer=KrylovDefaults.maxiter[],
+                  tol::Real=KrylovDefaults.tol[],
+                  verbosity::Int=KrylovDefaults.verbosity[])
     return BiCGStab(maxiter, tol, verbosity)
 end
 
+# Solving least squares problems
+abstract type LeastSquaresSolver <: KrylovAlgorithm end
+"""
+    LSMR(; krylovdim=1,
+            maxiter=KrylovDefaults.maxiter[],
+            tol=KrylovDefaults.tol[], 
+            orth::Orthogonalizer=ModifiedGramSchmidt(),
+            verbosity=KrylovDefaults.verbosity[])
+
+Represents the LSMR algorithm, which minimizes ``\\|Ax - b\\|^2 + \\|λx\\|^2`` in the Euclidean norm.
+If multiple solutions exists the minimum norm solution is returned.
+The method is based on the Golub-Kahan bidiagonalization process. It is
+algebraically equivalent to applying MINRES to the normal equations
+``(A^*A + λ^2I)x = A^*b``, but has better numerical properties,
+especially if ``A`` is ill-conditioned.
+
+The `LSMR` method will search for the optimal ``x`` in a Krylov subspace of maximal size 
+`maxiter`, or stop when ``norm(A'*(A*x - b) + λ^2 * x) < tol``. The parameter `krylovdim`
+does in this case not indicate that a subspace of that size will be built, but represents the
+number of most recent vectors that will be kept to which the next vector will be reorthogonalized.
+The default verbosity level `verbosity` amounts to printing warnings upon lack of convergence.
+
+See also: [`lssolve`](@ref)
+"""
+struct LSMR{O<:Orthogonalizer,S<:Real} <: LeastSquaresSolver
+    orth::O
+    maxiter::Int
+    krylovdim::Int
+    tol::S
+    verbosity::Int
+end
+function LSMR(;
+              krylovdim::Integer=KrylovDefaults.krylovdim[],
+              maxiter::Integer=KrylovDefaults.maxiter[],
+              tol::Real=KrylovDefaults.tol[],
+              orth::Orthogonalizer=ModifiedGramSchmidt(),
+              verbosity::Int=KrylovDefaults.verbosity[])
+    return LSMR(orth, maxiter, krylovdim, tol, verbosity)
+end
+
 # Solving eigenvalue systems specifically
 abstract type EigenSolver <: KrylovAlgorithm end
 
@@ -370,9 +436,10 @@ struct JacobiDavidson <: EigenSolver end
 """
     module KrylovDefaults
         const orth = KrylovKit.ModifiedGramSchmidtIR()
-        const krylovdim = 30
-        const maxiter = 100
-        const tol = 1e-12
+        const krylovdim = Ref(30)
+        const maxiter = Ref(100)
+        const tol = Ref(1e-12)
+        const verbosity = Ref(KrylovKit.WARN_LEVEL)
     end
 
 A module listing the default values for the typical parameters in Krylov based algorithms:
@@ -394,7 +461,8 @@ A module listing the default values for the typical parameters in Krylov based a
 module KrylovDefaults
 using ..KrylovKit
 const orth = KrylovKit.ModifiedGramSchmidt2() # conservative choice
-const krylovdim = 30
-const maxiter = 100
-const tol = 1e-12
+const krylovdim = Ref(30)
+const maxiter = Ref(100)
+const tol = Ref(1e-12)
+const verbosity = Ref(KrylovKit.WARN_LEVEL)
 end
