@@ -58,7 +58,7 @@ currently available.
 The return value is always of the form `T, vecs, vals, info = schursolve(...)` with
 
   - `T`: a `Matrix` containing the partial Schur decomposition of the linear map, i.e. it's
-    elements are given by `T[i,j] = dot(vecs[i], f(vecs[j]))`. It is of Schur form, i.e.
+    elements are given by `T[i,j] = inner(vecs[i], f(vecs[j]))`. It is of Schur form, i.e.
     upper triangular in case of complex arithmetic, and block upper triangular (with at most
     2x2 blocks) in case of real arithmetic.
   - `vecs`: a `Vector` of corresponding Schur vectors, of the same length as `vals`. Note
@@ -67,7 +67,7 @@ The return value is always of the form `T, vecs, vals, info = schursolve(...)` w
     objects that are typically similar to the starting guess `x₀`, up to a possibly
     different `eltype`. When the linear map is a simple `AbstractMatrix`, `vecs` will be
     `Vector{Vector{<:Number}}`. Schur vectors are by definition orthogonal, i.e.
-    `dot(vecs[i],vecs[j]) = I[i,j]`. Note that Schur vectors are real if the problem (i.e.
+    `inner(vecs[i],vecs[j]) = I[i,j]`. Note that Schur vectors are real if the problem (i.e.
     the linear map and the initial guess) are real.
   - `vals`: a `Vector` of eigenvalues, i.e. the diagonal elements of `T` in case of complex
     arithmetic, or extracted from the diagonal blocks in case of real arithmetic. Note that
@@ -107,32 +107,36 @@ restarts where a part of the current Krylov subspace is kept.
 """
 function schursolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi)
     T, U, fact, converged, numiter, numops = _schursolve(A, x₀, howmany, which, alg)
+    howmany′ = howmany
     if eltype(T) <: Real && howmany < length(fact) && T[howmany + 1, howmany] != 0
-        howmany += 1
+        howmany′ += 1
+    elseif size(T, 1) < howmany
+        howmany′ = size(T, 1)
     end
     if converged > howmany
-        howmany = converged
+        howmany′ = converged
     end
-    TT = view(T, 1:howmany, 1:howmany)
+    TT = view(T, 1:howmany′, 1:howmany′)
     values = schur2eigvals(TT)
+
     vectors = let B = basis(fact)
-        [B * u for u in cols(U, 1:howmany)]
+        [B * u for u in cols(U, 1:howmany′)]
     end
     residuals = let r = residual(fact)
-        [scale(r, last(u)) for u in cols(U, 1:howmany)]
+        [scale(r, last(u)) for u in cols(U, 1:howmany′)]
     end
-    normresiduals = [normres(fact) * abs(last(u)) for u in cols(U, 1:howmany)]
+    normresiduals = [normres(fact) * abs(last(u)) for u in cols(U, 1:howmany′)]
 
-    if alg.verbosity > 0
-        if converged < howmany
-            @warn """Arnoldi schursolve finished without convergence after $numiter iterations:
-             *  $converged eigenvalues converged
-             *  norm of residuals = $((normresiduals...,))"""
-        else
-            @info """Arnoldi schursolve finished after $numiter iterations:
-             *  $converged eigenvalues converged
-             *  norm of residuals = $((normresiduals...,))"""
-        end
+    if (converged < howmany) && alg.verbosity >= WARN_LEVEL
+        @warn """Arnoldi schursolve stopped without convergence after $numiter iterations:
+        * $converged eigenvalues converged
+        * norm of residuals = $(normres2string(normresiduals))
+        * number of operations = $numops"""
+    elseif alg.verbosity >= STARTSTOP_LEVEL
+        @info """Arnoldi schursolve finished after $numiter iterations:
+        * $converged eigenvalues converged
+        * norm of residuals = $(normres2string(normresiduals))
+        * number of operations = $numops"""
     end
     return TT,
            vectors,
@@ -142,18 +146,20 @@ end
 
 function eigsolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi; alg_rrule=alg)
     T, U, fact, converged, numiter, numops = _schursolve(A, x₀, howmany, which, alg)
+    howmany′ = howmany
     if eltype(T) <: Real && howmany < length(fact) && T[howmany + 1, howmany] != 0
-        howmany += 1
+        howmany′ += 1
+    elseif size(T, 1) < howmany
+        howmany′ = size(T, 1)
     end
     if converged > howmany
-        howmany = converged
+        howmany′ = converged
     end
-    d = min(howmany, size(T, 2))
-    TT = view(T, 1:d, 1:d)
+    TT = view(T, 1:howmany′, 1:howmany′)
     values = schur2eigvals(TT)
 
     # Compute eigenvectors
-    V = view(U, :, 1:d) * schur2eigvecs(TT)
+    V = view(U, :, 1:howmany′) * schur2eigvecs(TT)
     vectors = let B = basis(fact)
         [B * v for v in cols(V)]
     end
@@ -162,18 +168,16 @@ function eigsolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi; alg_rrul
     end
     normresiduals = [normres(fact) * abs(last(v)) for v in cols(V)]
 
-    if alg.verbosity > 0
-        if converged < howmany
-            @warn """Arnoldi eigsolve finished without convergence after $numiter iterations:
-             *  $converged eigenvalues converged
-             *  norm of residuals = $((normresiduals...,))
-             *  number of operations = $numops"""
-        else
-            @info """Arnoldi eigsolve finished after $numiter iterations:
-             *  $converged eigenvalues converged
-             *  norm of residuals = $((normresiduals...,))
-             *  number of operations = $numops"""
-        end
+    if (converged < howmany) && alg.verbosity >= WARN_LEVEL
+        @warn """Arnoldi eigsolve stopped without convergence after $numiter iterations:
+        * $converged eigenvalues converged
+        * norm of residuals = $(normres2string(normresiduals))
+        * number of operations = $numops"""
+    elseif alg.verbosity >= STARTSTOP_LEVEL
+        @info """Arnoldi eigsolve finished after $numiter iterations:
+        * $converged eigenvalues converged
+        * norm of residuals = $(normres2string(normresiduals))
+        * number of operations = $numops"""
     end
     return values,
            vectors,
@@ -185,10 +189,28 @@ end
     realeigsolve(f, x₀, howmany, which, algorithm; alg_rrule=algorithm)
 
 Compute the first `howmany` eigenvalues (according to the order specified by `which`)
-from the real linear map encoded in the matrix `A` or by the function `f`, with the guarantee
-that these eigenvalues (and thus their associated eigenvectors) are real.
+from the real linear map encoded in the matrix `A` or by the function `f`, if it can be
+guaranteed that these eigenvalues (and thus their associated eigenvectors) are real. An
+error will be thrown if there are complex eigenvalues within the first `howmany` eigenvalues.
+
 Return eigenvalues, eigenvectors and a `ConvergenceInfo` structure.
 
+!!! note "Note about real linear maps"
+
+    A function `f` is said to implement a real linear map if it satisfies 
+    `f(add(x,y)) = add(f(x), f(y)` and `f(scale(x, α)) = scale(f(x), α)` for vectors `x`
+    and `y` and scalars `α::Real`. Note that this is possible even when the vectors are
+    represented using complex arithmetic. For example, the map `f=x-> x + conj(x)`
+    represents a real linear map that is not (complex) linear, as it does not satisfy
+    `f(scale(x, α)) = scale(f(x), α)` for complex scalars `α`. Note that complex linear
+    maps are always real linear maps and thus can be used in this context, if looking
+    specifically for real eigenvalues that they may have.
+
+    To interpret the vectors `x` and `y` as elements from a real vector space, the standard
+    inner product defined on them will be replaced with `real(inner(x,y))`. This has no
+    effect if the vectors `x` and `y` were represented using real arithmetic to begin with,
+    and allows to seemlessly use complex vectors as well.
+
 ### Arguments:
 
 The linear map can be an `AbstractMatrix` (dense or sparse) or a general function or
@@ -268,49 +290,57 @@ The return value is always of the form `vals, vecs, info = eigsolve(...)` with
     if `info.converged >= howmany`.
 """
 function realeigsolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi; alg_rrule=alg)
-    T, U, fact, converged, numiter, numops = _schursolve(A, x₀, howmany, which, alg)
-    if !(eltype(T) <: Real)
-        throw(ArgumentError("realeigsolve can only be used for real eigenvalue problems"))
-    else
-        allreal = true
-        for i in 1:(howmany < length(fact) ? howmany : howmany - 1)
-            if T[i + 1, i] != 0
-                allreal = false
+    T, U, fact, converged, numiter, numops = _schursolve(A, RealVec(x₀), howmany, which,
+                                                         alg)
+    i = 0
+    while i < howmany
+        i += 1
+        if i < length(fact)
+            if abs(T[i + 1, i]) > alg.tol && alg.verbosity >= WARN_LEVEL
+                impart = sqrt(-T[i + 1, i] * T[i, i + 1])
+                warnmsg = "2 x 2 Schur block at position $i and $(i + 1) detected, complex eigenvalues "
+                warnmsg *= "with imaginary part $impart will be ignored by setting T[i+1,i] = $(T[i+1,i]) to zero."
+                @warn warnmsg
+            end
+            T[i + 1, i] = 0
+        end
+    end
+    while i < converged
+        i += 1
+        if i < length(fact)
+            if abs(T[i + 1, i]) <= alg.tol
+                T[i + 1, i] = 0
+            else
+                i -= 1
                 break
             end
         end
-        allreal || throw(ArgumentError("not all first `howmany` eigenvalues are real"))
-    end
-    if converged > howmany
-        while howmany < converged && T[howmany + 1, howmany] == 0
-            howmany += 1
-        end
     end
+    howmany = i
+    converged = min(converged, howmany)
     TT = view(T, 1:howmany, 1:howmany)
     values = diag(TT)
 
     # Compute eigenvectors
     V = view(U, :, 1:howmany) * schur2realeigvecs(TT)
     vectors = let B = basis(fact)
-        [B * v for v in cols(V)]
+        [(B * v)[] for v in cols(V)]
     end
-    residuals = let r = residual(fact)
+    residuals = let r = residual(fact)[]
         [scale(r, last(v)) for v in cols(V)]
     end
     normresiduals = [normres(fact) * abs(last(v)) for v in cols(V)]
 
-    if alg.verbosity > 0
-        if converged < howmany
-            @warn """Arnoldi realeigsolve finished without convergence after $numiter iterations:
-             *  $converged eigenvalues converged
-             *  norm of residuals = $((normresiduals...,))
-             *  number of operations = $numops"""
-        else
-            @info """Arnoldi realeigsolve finished after $numiter iterations:
-             *  $converged eigenvalues converged
-             *  norm of residuals = $((normresiduals...,))
-             *  number of operations = $numops"""
-        end
+    if (converged < howmany) && alg.verbosity >= WARN_LEVEL
+        @warn """Arnoldi realeigsolve stopped without convergence after $numiter iterations:
+        * $converged eigenvalues converged
+        * norm of residuals = $(normres2string(normresiduals))
+        * number of operations = $numops"""
+    elseif alg.verbosity >= STARTSTOP_LEVEL
+        @info """Arnoldi realeigsolve finished after $numiter iterations:
+        * $converged eigenvalues converged
+        * norm of residuals = $(normres2string(normresiduals))
+        * number of operations = $numops"""
     end
     return values,
            vectors,
@@ -327,7 +357,7 @@ function _schursolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi)
     numiter = 1
     # initialize arnoldi factorization
     iter = ArnoldiIterator(A, x₀, alg.orth)
-    fact = initialize(iter; verbosity=alg.verbosity - 2)
+    fact = initialize(iter; verbosity=alg.verbosity)
     numops = 1
     sizehint!(fact, krylovdim)
     β = normres(fact)
@@ -345,10 +375,11 @@ function _schursolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi)
         β = normres(fact)
         K = length(fact)
 
-        if β <= tol
-            if K < howmany
-                @warn "Invariant subspace of dimension $K (up to requested tolerance `tol = $tol`), which is smaller than the number of requested eigenvalues (i.e. `howmany == $howmany`); setting `howmany = $K`."
-                howmany = K
+        if β <= tol && K < howmany
+            if alg.verbosity >= WARN_LEVEL
+                msg = "Invariant subspace of dimension $K (up to requested tolerance `tol = $tol`), "
+                msg *= "which is smaller than the number of requested eigenvalues (i.e. `howmany == $howmany`)."
+                @warn msg
             end
         end
         if K == krylovdim || β <= tol || (alg.eager && K >= howmany) # process
@@ -374,33 +405,34 @@ function _schursolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi)
                 converged -= 1
             end
 
-            if converged >= howmany
+            if converged >= howmany || β <= tol
                 break
-            elseif alg.verbosity > 1
-                msg = "Arnoldi schursolve in iter $numiter, krylovdim = $K: "
-                msg *= "$converged values converged, normres = ("
-                msg *= @sprintf("%.2e", abs(f[1]))
-                for i in 2:howmany
-                    msg *= ", "
-                    msg *= @sprintf("%.2e", abs(f[i]))
-                end
-                msg *= ")"
-                @info msg
+            elseif alg.verbosity >= EACHITERATION_LEVEL
+                @info "Arnoldi schursolve in iteration $numiter, step = $K: $converged values converged, normres = $(normres2string(abs.(f[1:howmany])))"
             end
         end
 
         if K < krylovdim # expand
-            fact = expand!(iter, fact; verbosity=alg.verbosity - 2)
+            fact = expand!(iter, fact; verbosity=alg.verbosity)
             numops += 1
         else # shrink
             numiter == maxiter && break
 
             # Determine how many to keep
             keep = div(3 * krylovdim + 2 * converged, 5) # strictly smaller than krylovdim since converged < howmany <= krylovdim, at least equal to converged
-            if eltype(H) <: Real && H[keep + 1, keep] != 0 # we are in the middle of a 2x2 block
-                keep += 1 # conservative choice
-                keep >= krylovdim &&
-                    error("krylov dimension $(krylovdim) too small to compute $howmany eigenvalues")
+            if eltype(H) <: Real && H[keep + 1, keep] != 0
+                # we are in the middle of a 2x2 block; this cannot happen if keep == converged, so we can decrease keep
+                # however, we have to make sure that we do not end up with keep = 0
+                if keep > 1
+                    keep -= 1 # conservative choice
+                else
+                    keep += 1
+                    if krylovdim == 2
+                        alg.verbosity >= WARN_LEVEL &&
+                            @warn "Arnoldi iteration got stuck in a 2x2 block, consider increasing the Krylov dimension"
+                        break
+                    end
+                end
             end
 
             # Restore Arnoldi form in the first keep columns
@@ -424,7 +456,7 @@ function _schursolve(A, x₀, howmany::Int, which::Selector, alg::Arnoldi)
             B[keep + 1] = scale!!(r, 1 / normres(fact))
 
             # Shrink Arnoldi factorization
-            fact = shrink!(fact, keep)
+            fact = shrink!(fact, keep; verbosity=alg.verbosity)
             numiter += 1
         end
     end
